@@ -1,14 +1,27 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
-/*
- * Author:
- *   Chris Toshok (toshok@ximian.com)
+
+/* e-book-backend-vcf.c - VCF contact backend.
  *
- * Copyright (C) 2003, Ximian, Inc.
+ * Copyright (C) 2005 Novell, Inc.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of version 2 of the GNU Lesser General Public
+ * License as published by the Free Software Foundation.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this program; if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
+ *
+ * Authors: Chris Toshok <toshok@ximian.com>
  */
 
-#ifdef HAVE_CONFIG_H
 #include <config.h>
-#endif
 
 #include <stdio.h>
 #include <string.h>
@@ -17,21 +30,31 @@
 #include <fcntl.h>
 #include <time.h>
 #include <sys/stat.h>
+#include <sys/time.h>
 
-#include <libgnome/gnome-i18n.h>
-#include <libebook/e-contact.h>
+#ifndef O_BINARY
+#define O_BINARY 0
+#endif
 
-#include <libedataserver/e-util.h>
+#include <glib.h>
+#include <glib/gstdio.h>
+#include <glib/gi18n-lib.h>
+
+#include "libedataserver/e-data-server-util.h"
+#include "libedataserver/e-flag.h"
+
+#include "libebook/e-contact.h"
  
-#include <libedata-book/e-data-book.h>
-#include <libedata-book/e-data-book-view.h>
-#include <libedata-book/e-book-backend-sexp.h>
+#include "libedata-book/e-data-book.h"
+#include "libedata-book/e-data-book-view.h"
+#include "libedata-book/e-book-backend-sexp.h"
+
 #include "e-book-backend-vcf.h"
 
 #define PAS_ID_PREFIX "pas-id-"
 #define FILE_FLUSH_TIMEOUT 5000
 
-#define d(x)
+#define d(x) x
 
 static EBookBackendSyncClass *e_book_backend_vcf_parent_class;
 typedef struct _EBookBackendVCFBookView EBookBackendVCFBookView;
@@ -41,8 +64,14 @@ struct _EBookBackendVCFPrivate {
 	char       *filename;
 	GMutex     *mutex;
 	GHashTable *contacts;
+	GList      *contact_list;
 	gboolean    dirty;
 	int         flush_timeout_tag;
+	/* for future use */
+	void *reserved1;
+	void *reserved2;
+	void *reserved3;
+	void *reserved4;
 };
 
 static char *
@@ -62,10 +91,15 @@ insert_contact (EBookBackendVCF *vcf, char *vcard)
 	char *id;
 
 	id = e_contact_get (contact, E_CONTACT_UID);
-	if (id)
+	if (id) {
+		char *vcard = e_vcard_to_string (E_VCARD (contact), EVC_FORMAT_VCARD_30);
+
+		vcf->priv->contact_list = g_list_prepend (vcf->priv->contact_list, vcard);
+
 		g_hash_table_insert (vcf->priv->contacts,
 				     id,
-				     e_vcard_to_string (E_VCARD (contact), EVC_FORMAT_VCARD_30));
+				     vcf->priv->contact_list);
+	}
 }
 
 static void
@@ -75,8 +109,9 @@ load_file (EBookBackendVCF *vcf, int fd)
 	GString *str;
 	char buf[1024];
 
-	fp = fdopen (fd, "r");
+	fp = fdopen (fd, "rb");
 	if (!fp) {
+		close (fd); /* callers depend on fd being closed by this function */
 		g_warning ("failed to open `%s' for reading", vcf->priv->filename);
 		return;
 	}
@@ -104,16 +139,10 @@ load_file (EBookBackendVCF *vcf, int fd)
 	fclose (fp);
 }
 
-static void
-foreach_build_list (char *id, char *vcard_string, GList **list)
-{
-	*list = g_list_append (*list, vcard_string);
-}
-
 static gboolean
 save_file (EBookBackendVCF *vcf)
 {
-	GList *vcards = NULL;
+	gboolean retv = FALSE;
 	GList *l;
 	char *new_path;
 	int fd, rv;
@@ -121,13 +150,16 @@ save_file (EBookBackendVCF *vcf)
 	g_warning ("EBookBackendVCF flushing file to disk");
 
 	g_mutex_lock (vcf->priv->mutex);
-	g_hash_table_foreach (vcf->priv->contacts, (GHFunc)foreach_build_list, &vcards);
 
 	new_path = g_strdup_printf ("%s.new", vcf->priv->filename);
 
-	fd = open (new_path, O_CREAT | O_TRUNC | O_WRONLY, 0666);
+	fd = g_open (new_path, O_CREAT | O_TRUNC | O_WRONLY | O_BINARY, 0666);
+	if (fd == -1) {
+		g_warning ("write failed.  could not open output file\n");
+		goto out;
+	}
 
-	for (l = vcards; l; l = l->next) {
+	for (l = vcf->priv->contact_list; l; l = l->next) {
 		char *vcard_str = l->data;
 		int len = strlen (vcard_str);
 
@@ -136,35 +168,34 @@ save_file (EBookBackendVCF *vcf)
 		if (rv < len) {
 			/* XXX */
 			g_warning ("write failed.  we need to handle short writes\n");
-			close (fd);
-			unlink (new_path);
-			return FALSE;
+			g_unlink (new_path);
+			goto out;
 		}
 
 		rv = write (fd, "\r\n\r\n", 4);
 		if (rv < 4) {
 			/* XXX */
 			g_warning ("write failed.  we need to handle short writes\n");
-			close (fd);
-			unlink (new_path);
-			return FALSE;
+			g_unlink (new_path);
+			goto out;
 		}
 	}
 
-	if (0 > rename (new_path, vcf->priv->filename)) {
+	if (0 > g_rename (new_path, vcf->priv->filename)) {
 		g_warning ("Failed to rename %s: %s\n", vcf->priv->filename, strerror(errno));
-		unlink (new_path);
-		return FALSE;
+		g_unlink (new_path);
+		goto out;
 	}
+	retv = TRUE;
 
-	g_list_free (vcards);
+out:
+	if (fd != -1)
+		close (fd);
 	g_free (new_path);
-
-	vcf->priv->dirty = FALSE;
-
+	vcf->priv->dirty = !retv;
 	g_mutex_unlock (vcf->priv->mutex);
 
-	return TRUE;
+	return retv;
 }
 
 static gboolean
@@ -186,6 +217,20 @@ vcf_flush_file (gpointer data)
 	return FALSE;
 }
 
+static void
+set_revision (EContact *contact)
+{
+	char time_string[25] = {0};
+	const struct tm *tm = NULL;
+	GTimeVal tv;
+
+	g_get_current_time (&tv);
+	tm = gmtime (&tv.tv_sec);
+	if (tm)
+		strftime (time_string, 100, "%Y-%m-%dT%H:%M:%SZ", tm);
+	e_contact_set (contact, E_CONTACT_REV, time_string);
+}
+
 static EContact *
 do_create(EBookBackendVCF  *bvcf,
 	  const char     *vcard_req,
@@ -194,6 +239,7 @@ do_create(EBookBackendVCF  *bvcf,
 	char           *id;
 	EContact       *contact;
 	char           *vcard;
+	const char     *rev;
 
 	/* at the very least we need the unique_id generation to be
 	   protected by the lock, even if the actual vcard parsing
@@ -203,9 +249,15 @@ do_create(EBookBackendVCF  *bvcf,
 
 	contact = e_contact_new_from_vcard (vcard_req);
 	e_contact_set(contact, E_CONTACT_UID, id);
+	g_free (id);
+
+	rev = e_contact_get_const (contact,  E_CONTACT_REV);
+	if (!(rev && *rev))
+		set_revision (contact);
+
 	vcard = e_vcard_to_string (E_VCARD (contact), EVC_FORMAT_VCARD_30);
 
-	g_hash_table_insert (bvcf->priv->contacts, id, vcard);
+	insert_contact (bvcf, vcard);
 
 	if (dirty_the_file) {
 		bvcf->priv->dirty = TRUE;
@@ -223,6 +275,7 @@ do_create(EBookBackendVCF  *bvcf,
 static EBookBackendSyncStatus
 e_book_backend_vcf_create_contact (EBookBackendSync *backend,
 				   EDataBook *book,
+				   guint32 opid,
 				   const char *vcard,
 				   EContact **contact)
 {
@@ -241,43 +294,56 @@ e_book_backend_vcf_create_contact (EBookBackendSync *backend,
 
 static EBookBackendSyncStatus
 e_book_backend_vcf_remove_contacts (EBookBackendSync *backend,
-				    EDataBook    *book,
+				    EDataBook *book,
+				    guint32 opid,
 				    GList *id_list,
 				    GList **ids)
 {
 	/* FIXME: make this handle bulk deletes like the file backend does */
 	EBookBackendVCF *bvcf = E_BOOK_BACKEND_VCF (backend);
 	char *id = id_list->data;
+	GList *elem;
 	gboolean success;
+	char *real_id;
 
 	g_mutex_lock (bvcf->priv->mutex);
-	success = g_hash_table_remove (bvcf->priv->contacts, id);
-
+	success = g_hash_table_lookup_extended (bvcf->priv->contacts, id, (gpointer)&real_id, (gpointer)&elem);
 	if (!success) {
 		g_mutex_unlock (bvcf->priv->mutex);
 		return GNOME_Evolution_Addressbook_ContactNotFound;
 	}
-	else {
-		bvcf->priv->dirty = TRUE;
-		if (!bvcf->priv->flush_timeout_tag)
-			bvcf->priv->flush_timeout_tag = g_timeout_add (FILE_FLUSH_TIMEOUT,
-								       vcf_flush_file, bvcf);
+
+	success = g_hash_table_remove (bvcf->priv->contacts, real_id);
+	if (!success) {
 		g_mutex_unlock (bvcf->priv->mutex);
-
-		*ids = g_list_append (*ids, id);
-
-		return GNOME_Evolution_Addressbook_Success;
+		return GNOME_Evolution_Addressbook_ContactNotFound;
 	}
+
+	g_free (real_id);
+	g_free (elem->data);
+	bvcf->priv->contact_list = g_list_remove_link (bvcf->priv->contact_list, elem);
+
+	bvcf->priv->dirty = TRUE;
+	if (!bvcf->priv->flush_timeout_tag)
+		bvcf->priv->flush_timeout_tag = g_timeout_add (FILE_FLUSH_TIMEOUT,
+							       vcf_flush_file, bvcf);
+	g_mutex_unlock (bvcf->priv->mutex);
+
+	*ids = g_list_append (*ids, id);
+
+	return GNOME_Evolution_Addressbook_Success;
 }
 
 static EBookBackendSyncStatus
 e_book_backend_vcf_modify_contact (EBookBackendSync *backend,
 				   EDataBook *book,
+				   guint32 opid,
 				   const char *vcard,
 				   EContact **contact)
 {
 	EBookBackendVCF *bvcf = E_BOOK_BACKEND_VCF (backend);
-	char *old_id, *old_vcard_string;
+	char *old_id;
+	GList *elem;
 	const char *id;
 	gboolean success;
 
@@ -286,39 +352,38 @@ e_book_backend_vcf_modify_contact (EBookBackendSync *backend,
 	id = e_contact_get_const (*contact, E_CONTACT_UID);
 
 	g_mutex_lock (bvcf->priv->mutex);
-	success = g_hash_table_lookup_extended (bvcf->priv->contacts, id, (gpointer)&old_id, (gpointer)&old_vcard_string);
+	success = g_hash_table_lookup_extended (bvcf->priv->contacts, id, (gpointer)&old_id, (gpointer)&elem);
 
 	if (!success) {
 		g_mutex_unlock (bvcf->priv->mutex);
 		return GNOME_Evolution_Addressbook_ContactNotFound;
 	}
-	else {
-		g_hash_table_insert (bvcf->priv->contacts, old_id, g_strdup (vcard));
-		bvcf->priv->dirty = TRUE;
-		if (!bvcf->priv->flush_timeout_tag)
-			bvcf->priv->flush_timeout_tag = g_timeout_add (FILE_FLUSH_TIMEOUT,
-								       vcf_flush_file, bvcf);
-		g_mutex_unlock (bvcf->priv->mutex);
 
-		g_free (old_vcard_string);
+	g_free (elem->data);
+	elem->data = g_strdup (vcard);
+	bvcf->priv->dirty = TRUE;
+	if (!bvcf->priv->flush_timeout_tag)
+		bvcf->priv->flush_timeout_tag = g_timeout_add (FILE_FLUSH_TIMEOUT,
+							       vcf_flush_file, bvcf);
+	g_mutex_unlock (bvcf->priv->mutex);
 
-		return GNOME_Evolution_Addressbook_Success;
-	}
+	return GNOME_Evolution_Addressbook_Success;
 }
 
 static EBookBackendSyncStatus
 e_book_backend_vcf_get_contact (EBookBackendSync *backend,
-				EDataBook    *book,
+				EDataBook *book,
+				guint32 opid,
 				const char *id,
 				char **vcard)
 {
 	EBookBackendVCF *bvcf = E_BOOK_BACKEND_VCF (backend);
-	char *v;
+	GList *elem;
 
-	v = g_hash_table_lookup (bvcf->priv->contacts, id);
+	elem = g_hash_table_lookup (bvcf->priv->contacts, id);
 
-	if (v) {
-		*vcard = g_strdup (v);
+	if (elem) {
+		*vcard = g_strdup (elem->data);
 		return GNOME_Evolution_Addressbook_Success;
 	} else {
 		*vcard = g_strdup ("");
@@ -335,7 +400,7 @@ typedef struct {
 } GetContactListClosure;
 
 static void
-foreach_get_contact_compare (char *id, char *vcard_string, GetContactListClosure *closure)
+foreach_get_contact_compare (char *vcard_string, GetContactListClosure *closure)
 {
 	if ((!closure->search_needed) || e_book_backend_sexp_match_vcard  (closure->card_sexp, vcard_string)) {
 		closure->list = g_list_append (closure->list, g_strdup (vcard_string));
@@ -344,7 +409,8 @@ foreach_get_contact_compare (char *id, char *vcard_string, GetContactListClosure
 
 static EBookBackendSyncStatus
 e_book_backend_vcf_get_contact_list (EBookBackendSync *backend,
-				     EDataBook    *book,
+				     EDataBook *book,
+				     guint32 opid,
 				     const char *query,
 				     GList **contacts)
 {
@@ -357,7 +423,7 @@ e_book_backend_vcf_get_contact_list (EBookBackendSync *backend,
 	closure.card_sexp = e_book_backend_sexp_new (search);
 	closure.list = NULL;
 
-	g_hash_table_foreach (bvcf->priv->contacts, (GHFunc)foreach_get_contact_compare, &closure);
+	g_list_foreach (bvcf->priv->contact_list, (GFunc)foreach_get_contact_compare, &closure);
 
 	g_object_unref (closure.card_sexp);
 
@@ -366,29 +432,29 @@ e_book_backend_vcf_get_contact_list (EBookBackendSync *backend,
 }
 
 typedef struct {
-	GMutex *mutex;
-	gboolean stopped;
 	EBookBackendVCF *bvcf;
 	EDataBookView *view;
+	GThread *thread;
+	EFlag *running;
 } VCFBackendSearchClosure;
 
 static void
 closure_destroy (VCFBackendSearchClosure *closure)
 {
 	d(printf ("destroying search closure\n"));
-	g_mutex_free (closure->mutex);
+	e_flag_free (closure->running);
 	g_free (closure);
 }
 
 static VCFBackendSearchClosure*
-init_closure (EDataBookView *book_view)
+init_closure (EDataBookView *book_view, EBookBackendVCF *bvcf)
 {
 	VCFBackendSearchClosure *closure = g_new (VCFBackendSearchClosure, 1);
 
-	closure->mutex = g_mutex_new();
+	closure->bvcf = bvcf;
 	closure->view = book_view;
-	closure->bvcf = E_BOOK_BACKEND_VCF (e_data_book_view_get_backend (book_view));
-	closure->stopped = FALSE;
+	closure->thread = NULL;
+	closure->running = e_flag_new ();
 
 	g_object_set_data_full (G_OBJECT (book_view), "EBookBackendVCF.BookView::closure",
 				closure, (GDestroyNotify)closure_destroy);
@@ -399,109 +465,96 @@ init_closure (EDataBookView *book_view)
 static VCFBackendSearchClosure*
 get_closure (EDataBookView *book_view)
 {
-	return g_object_get_data (G_OBJECT (book_view), "EBookBackendFile.BookView::closure");
+	return g_object_get_data (G_OBJECT (book_view), "EBookBackendVCF.BookView::closure");
 }
 
-static void
-foreach_search_compare (char *id, char *vcard_string, VCFBackendSearchClosure *closure)
+static gpointer
+book_view_thread (gpointer data)
 {
-	EContact *contact;
-
-	/* we really need to stop the entire loop when stopped ==
-	   TRUE, but we can't loop over the hash table that way..
-	   lame.  so we just return here. */
-	if (closure->stopped)
-		return;
-
-	contact = e_contact_new_from_vcard (vcard_string);
-	e_data_book_view_notify_update (closure->view, contact);
-	g_object_unref (contact);
-}
-
-static void
-e_book_backend_vcf_start_book_view (EBookBackend  *backend,
-				    EDataBookView *book_view)
-{
-	VCFBackendSearchClosure *closure;
-	const char *query = e_data_book_view_get_card_query (book_view);
-
-	g_mutex_lock (e_data_book_view_get_mutex (book_view));
-
-	closure = get_closure (book_view);
-
-	if (closure) {
-		if (!closure->stopped) {
-			g_warning ("lost race with stop_book_view, but op->stopped != TRUE");
-		}
-		g_object_set_data (G_OBJECT (book_view), "EBookBackendVCF.BookView::closure", NULL);
-		g_mutex_unlock (e_data_book_view_get_mutex (book_view));
-		return;
-	}
-	else {
-		closure = init_closure (book_view);
-		closure->stopped = FALSE;
-	}
+	EDataBookView *book_view = data;
+	VCFBackendSearchClosure *closure = get_closure (book_view);
+	const char *query;
+	GList *l;
 
 	/* ref the book view because it'll be removed and unrefed
 	   when/if it's stopped */
 	bonobo_object_ref (book_view);
 
-	g_mutex_unlock (e_data_book_view_get_mutex (book_view));
+	query = e_data_book_view_get_card_query (book_view);
 
 	if ( ! strcmp (query, "(contains \"x-evolution-any-field\" \"\")"))
 		e_data_book_view_notify_status_message (book_view, _("Loading..."));
 	else
 		e_data_book_view_notify_status_message (book_view, _("Searching..."));
 
-	g_hash_table_foreach (closure->bvcf->priv->contacts,
-			      (GHFunc)foreach_search_compare,
-			      closure);
+	d(printf ("signalling parent thread\n"));
+	e_flag_set (closure->running);
 
-	if (!closure->stopped)
+	for (l = closure->bvcf->priv->contact_list; l; l = l->next) {
+		char *vcard_string = l->data;
+		EContact *contact = e_contact_new_from_vcard (vcard_string);
+		e_data_book_view_notify_update (closure->view, contact);
+		g_object_unref (contact);
+
+		if (!e_flag_is_set (closure->running))
+			break;
+	}
+
+	if (e_flag_is_set (closure->running))
 		e_data_book_view_notify_complete (closure->view, GNOME_Evolution_Addressbook_Success);
 
 	/* unref the book view */
 	bonobo_object_unref (book_view);
 
 	d(printf ("finished initial population of book view\n"));
+
+	return NULL;
+}
+
+
+static void
+e_book_backend_vcf_start_book_view (EBookBackend  *backend,
+				    EDataBookView *book_view)
+{
+	VCFBackendSearchClosure *closure = init_closure (book_view, E_BOOK_BACKEND_VCF (backend));
+
+	d(printf ("starting book view thread\n"));
+	closure->thread = g_thread_create (book_view_thread, book_view, TRUE, NULL);
+
+	e_flag_wait (closure->running);
+
+	/* at this point we know the book view thread is actually running */
+	d(printf ("returning from start_book_view\n"));
+
 }
 
 static void
 e_book_backend_vcf_stop_book_view (EBookBackend  *backend,
 				   EDataBookView *book_view)
 {
-	VCFBackendSearchClosure *closure;
+	VCFBackendSearchClosure *closure = get_closure (book_view);
+	gboolean need_join;
 
-	g_mutex_lock (e_data_book_view_get_mutex (book_view));
+	d(printf ("stopping query\n"));
+	need_join = e_flag_is_set (closure->running);
+	e_flag_clear (closure->running);
 
-	closure = get_closure (book_view);
-
-	if (closure) {
-		d(printf ("stopping running query!\n"));
-		g_mutex_lock (closure->mutex);
-		closure->stopped = TRUE;
-		g_mutex_unlock (closure->mutex);
-	}
-	else {
-		d(printf ("either the query is already finished or hasn't started yet (we won the race)\n"));
-		closure = init_closure (book_view);
-		closure->stopped = TRUE;
-	}
-
-	g_mutex_unlock (e_data_book_view_get_mutex (book_view));
+	if (need_join)
+		g_thread_join (closure->thread);
 }
 
 static char *
 e_book_backend_vcf_extract_path_from_uri (const char *uri)
 {
-	g_assert (strncasecmp (uri, "vcf://", 6) == 0);
+	g_assert (g_ascii_strncasecmp (uri, "vcf://", 6) == 0);
 
 	return g_strdup (uri + 6);
 }
 
 static EBookBackendSyncStatus
 e_book_backend_vcf_authenticate_user (EBookBackendSync *backend,
-				      EDataBook    *book,
+				      EDataBook *book,
+				      guint32 opid,
 				      const char *user,
 				      const char *passwd,
 				      const char *auth_method)
@@ -509,9 +562,23 @@ e_book_backend_vcf_authenticate_user (EBookBackendSync *backend,
 	return GNOME_Evolution_Addressbook_Success;
 }
 
+static EBookBackendSyncStatus
+e_book_backend_vcf_get_required_fields (EBookBackendSync *backend,
+					 EDataBook *book,
+					 guint32 opid,
+					 GList **fields_out)
+{
+	GList *fields = NULL;
+
+	fields = g_list_append (fields , g_strdup(e_contact_field_name (E_CONTACT_FILE_AS)));
+       	*fields_out = fields;
+	return GNOME_Evolution_Addressbook_Success;
+}
+
 static EBookBackendSyncStatus
 e_book_backend_vcf_get_supported_fields (EBookBackendSync *backend,
-					 EDataBook    *book,
+					 EDataBook *book,
+					 guint32 opid,
 					 GList **fields_out)
 {
 	GList *fields = NULL;
@@ -526,7 +593,9 @@ e_book_backend_vcf_get_supported_fields (EBookBackendSync *backend,
 	return GNOME_Evolution_Addressbook_Success;
 }
 
-#include <libedata-book/ximian-vcard.h>
+#ifdef CREATE_DEFAULT_VCARD
+# include <libedata-book/ximian-vcard.h>
+#endif
 
 static GNOME_Evolution_Addressbook_CallStatus
 e_book_backend_vcf_load_source (EBookBackend             *backend,
@@ -544,22 +613,21 @@ e_book_backend_vcf_load_source (EBookBackend             *backend,
 	dirname = e_book_backend_vcf_extract_path_from_uri (uri);
 	bvcf->priv->filename = g_build_filename (dirname, "addressbook.vcf", NULL);
 
-	fd = open (bvcf->priv->filename, O_RDWR);
+	fd = g_open (bvcf->priv->filename, O_RDWR | O_BINARY, 0);
 
-	bvcf->priv->contacts = g_hash_table_new_full (g_str_hash, g_str_equal,
-						      g_free, g_free);
+	bvcf->priv->contacts = g_hash_table_new (g_str_hash, g_str_equal);
 
 	if (fd != -1) {
 		writable = TRUE;
 	} else {
-		fd = open (bvcf->priv->filename, O_RDONLY);
+		fd = g_open (bvcf->priv->filename, O_RDONLY | O_BINARY, 0);
 
 		if (fd == -1 && !only_if_exists) {
 			int rv;
 
 			/* the database didn't exist, so we create the
 			   directory then the .vcf file */
-			rv = e_util_mkdir_hier (dirname, 0777);
+			rv = g_mkdir_with_parents (dirname, 0777);
 			if (rv == -1 && errno != EEXIST) {
 				g_warning ("failed to make directory %s: %s", dirname, strerror (errno));
 				if (errno == EACCES || errno == EPERM)
@@ -568,9 +636,10 @@ e_book_backend_vcf_load_source (EBookBackend             *backend,
 					return GNOME_Evolution_Addressbook_OtherError;
 			}
 
-			fd = open (bvcf->priv->filename, O_CREAT, 0666);
+			fd = g_open (bvcf->priv->filename, O_CREAT | O_BINARY, 0666);
 
 			if (fd != -1) {
+#ifdef CREATE_DEFAULT_VCARD
 				EContact *contact;
 
 				contact = do_create(bvcf, XIMIAN_VCARD, FALSE);
@@ -578,6 +647,7 @@ e_book_backend_vcf_load_source (EBookBackend             *backend,
 
 				/* XXX check errors here */
 				g_object_unref (contact);
+#endif
 
 				writable = TRUE;
 			}
@@ -603,7 +673,7 @@ e_book_backend_vcf_load_source (EBookBackend             *backend,
 static char *
 e_book_backend_vcf_get_static_capabilities (EBookBackend *backend)
 {
-	return g_strdup("local,do-initial-query");
+	return g_strdup("local,do-initial-query,contact-lists");
 }
 
 static GNOME_Evolution_Addressbook_CallStatus
@@ -612,6 +682,15 @@ e_book_backend_vcf_cancel_operation (EBookBackend *backend, EDataBook *book)
 	return GNOME_Evolution_Addressbook_CouldNotCancel;
 }
 
+static void 
+e_book_backend_vcf_set_mode (EBookBackend *backend, int mode)
+{
+	if (e_book_backend_is_loaded (backend)) {
+		e_book_backend_notify_writable (backend, TRUE);
+		e_book_backend_notify_connection_status (backend, TRUE);
+	}
+}
+
 static gboolean
 e_book_backend_vcf_construct (EBookBackendVCF *backend)
 {
@@ -662,7 +741,10 @@ e_book_backend_vcf_dispose (GObject *object)
 		if (bvcf->priv->dirty)
 			save_file (bvcf);
 
+		g_hash_table_foreach (bvcf->priv->contacts, (GHFunc)g_free, NULL);
 		g_hash_table_destroy (bvcf->priv->contacts);
+		g_list_foreach (bvcf->priv->contact_list, (GFunc)g_free, NULL);
+		g_list_free (bvcf->priv->contact_list);
 
 		g_free (bvcf->priv->filename);
 
@@ -695,13 +777,14 @@ e_book_backend_vcf_class_init (EBookBackendVCFClass *klass)
 	backend_class->start_book_view         = e_book_backend_vcf_start_book_view;
 	backend_class->stop_book_view          = e_book_backend_vcf_stop_book_view;
 	backend_class->cancel_operation        = e_book_backend_vcf_cancel_operation;
-
+	backend_class->set_mode                = e_book_backend_vcf_set_mode;
 	sync_class->create_contact_sync        = e_book_backend_vcf_create_contact;
 	sync_class->remove_contacts_sync       = e_book_backend_vcf_remove_contacts;
 	sync_class->modify_contact_sync        = e_book_backend_vcf_modify_contact;
 	sync_class->get_contact_sync           = e_book_backend_vcf_get_contact;
 	sync_class->get_contact_list_sync      = e_book_backend_vcf_get_contact_list;
 	sync_class->authenticate_user_sync     = e_book_backend_vcf_authenticate_user;
+	sync_class->get_required_fields_sync   = e_book_backend_vcf_get_required_fields;
 	sync_class->get_supported_fields_sync  = e_book_backend_vcf_get_supported_fields;
 
 	object_class->dispose = e_book_backend_vcf_dispose;
