@@ -197,6 +197,7 @@ typedef enum {
 
 	E_CONTACT_IM_GADUGADU,   /* Multi-valued */
 	
+	E_CONTACT_GEO,	/* structured field (EContactGeo) */	
 
 	E_CONTACT_FIELD_LAST,
 	E_CONTACT_FIELD_FIRST        = E_CONTACT_UID,
@@ -222,6 +223,11 @@ typedef struct {
 	char *suffixes;
 } EContactName;
 
+typedef struct {
+	double latitude;
+	double longitude;
+} EContactGeo;
+
 typedef enum {
 	E_CONTACT_PHOTO_TYPE_INLINED,
 	E_CONTACT_PHOTO_TYPE_URI
@@ -232,7 +238,7 @@ typedef struct {
 	union {
 		struct {
 			char *mime_type;
-			int length;
+			gsize length;
 			guchar *data;
 		} inlined;
 		char *uri;
@@ -253,13 +259,13 @@ typedef struct {
 } EContactAddress;
 
 typedef struct {
-	int year;
-	int month;
-	int day;
+	unsigned int year;
+	unsigned int month;
+	unsigned int day;
 } EContactDate;
 
 typedef struct {
-	int length;
+	gsize length;
 	char *data;
 } EContactCert;
 
@@ -309,13 +315,16 @@ GType                   e_contact_name_get_type    (void);
 EContactName           *e_contact_name_new         (void);
 char                   *e_contact_name_to_string   (const EContactName *name);
 EContactName           *e_contact_name_from_string (const char *name_str);
-EContactName           *e_contact_name_copy        (EContactName *name);
+EContactName           *e_contact_name_copy        (EContactName *n);
 void                    e_contact_name_free        (EContactName *name);
 
 
 GType                   e_contact_photo_get_type   (void);
 void                    e_contact_photo_free       (EContactPhoto *photo);
 
+GType			e_contact_geo_get_type	   (void);
+void			e_contact_geo_free	   (EContactGeo *geo);
+
 GType                   e_contact_cert_get_type    (void);
 void                    e_contact_cert_free        (EContactCert *cert);
 
