@@ -3,7 +3,7 @@
  FILE: icaltimezone.c
  CREATOR: Damon Chaplin 15 March 2001
 
- $Id: icaltimezone.c 565 2006-06-16 10:33:12Z tml $
+ $Id: icaltimezone.c 615 2007-08-09 11:42:17Z pchen $
  $Locker$
 
  (C) COPYRIGHT 2001, Damon Chaplin
@@ -30,6 +30,7 @@
 #include "config.h"
 #endif
 
+#include <ctype.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -38,6 +39,7 @@
 #include "icalerror.h"
 #include "icalparser.h"
 #include "icaltimezone.h"
+#include "icaltz-util.h"
 
 #ifdef WIN32
 #include <mbstring.h>
@@ -64,6 +66,12 @@
     coordinates of all the builtin timezones. */
 #define ZONES_TAB_FILENAME	"zones.tab"
 
+#if defined(sun) && defined(__SVR4)
+#define ZONES_TAB_SYSTEM_FILENAME "zone_sun.tab"
+#else
+#define ZONES_TAB_SYSTEM_FILENAME "zone.tab"
+#endif
+
 /** This is the number of years of extra coverage we do when expanding
     the timezone changes. */
 #define ICALTIMEZONE_EXTRA_COVERAGE	5
@@ -1341,7 +1349,7 @@ icaltimezone*
 icaltimezone_get_builtin_timezone	(const char *location)
 {
     icaltimezone *zone;
-    int lower, upper, middle, cmp;
+    int lower;
     char *zone_location;
 
     if (!location || !location[0])
@@ -1352,7 +1360,8 @@ icaltimezone_get_builtin_timezone	(const char *location)
 
     if (!strcmp (location, "UTC"))
 	return &utc_timezone;
-
+    
+#if 0
     /* Do a simple binary search. */
     lower = middle = 0;
     upper = builtin_timezones->num_elements;
@@ -1369,6 +1378,16 @@ icaltimezone_get_builtin_timezone	(const char *location)
 	else
 	    lower = middle + 1;
     }
+#endif
+
+    /* The zones from the system are not stored in alphabetical order,
+       so we just do a sequential search */
+    for (lower = 0; lower < builtin_timezones->num_elements; lower++) {
+	zone = icalarray_element_at (builtin_timezones, lower);
+	zone_location = icaltimezone_get_location (zone);
+	if (strcmp (location, zone_location) == 0)
+		return zone;
+    }
 
     return NULL;
 }
@@ -1423,6 +1442,9 @@ icaltimezone_get_builtin_timezone_from_offset	(int offset, const char *tzname)
     if (offset==0)
 	return &utc_timezone;
 
+    if (!tzname)
+	return NULL;
+
     count = builtin_timezones->num_elements;
 
     for (i=0; i<count; i++) {
@@ -1433,7 +1455,7 @@ icaltimezone_get_builtin_timezone_from_offset	(int offset, const char *tzname)
 	
 	z_offset = get_offset(zone);
 
-	if (z_offset == offset && !strcmp(tzname, zone->tznames))
+	if (z_offset == offset && zone->tznames && !strcmp(tzname, zone->tznames))
 	    return zone;
     }
     
@@ -1445,8 +1467,7 @@ icaltimezone*
 icaltimezone_get_builtin_timezone_from_tzid (const char *tzid)
 {
     int num_slashes = 0;
-    const char *p, *zone_tzid;
-    icaltimezone *zone;
+    const char *p;
 
     if (!tzid || !tzid[0])
 	return NULL;
@@ -1471,18 +1492,8 @@ icaltimezone_get_builtin_timezone_from_tzid (const char *tzid)
     p++;
 
     /* Now we can use the function to get the builtin timezone from the
-       location string. */
-    zone = icaltimezone_get_builtin_timezone (p);
-    if (!zone)
-	return NULL;
-
-    /* Check that the builtin TZID matches exactly. We don't want to return
-       a different version of the VTIMEZONE. */
-    zone_tzid = icaltimezone_get_tzid (zone);
-    if (!strcmp (zone_tzid, tzid))
-	return zone;
-    else
-	return NULL;
+       location string and return it. */
+    return icaltimezone_get_builtin_timezone (p);
 }
 
 
@@ -1510,6 +1521,72 @@ icaltimezone_init_builtin_timezones	(void)
     icaltimezone_parse_zone_tab ();
 }
 
+static int
+parse_coord			(char		*coord,
+				 int		 len,
+				 int		*degrees, 
+				 int 		*minutes,
+				 int 		*seconds)
+{
+	if (len == 5)
+		sscanf (coord + 1, "%2d%2d", degrees, minutes);
+	else if (len == 6)
+		sscanf (coord + 1, "%3d%2d", degrees, minutes);
+	else if (len == 7)
+		sscanf (coord + 1, "%2d%2d%2d", degrees, minutes, seconds);
+	else if (len == 8)
+		sscanf (coord + 1, "%3d%2d%2d", degrees, minutes, seconds);
+	else {
+		fprintf (stderr, "Invalid coordinate: %s\n", coord);
+		return 1;
+	}
+
+	if (coord [0] == '-')
+		*degrees = -*degrees;
+	return 0;
+}
+static int
+fetch_lat_long_from_string  (const char *str, int *latitude_degrees, int *latitude_minutes, int *latitude_seconds,
+		int *longitude_degrees, int *longitude_minutes, int *longitude_seconds, char *location)
+{
+	size_t len;
+	char *sptr, *lat, *lon, *loc, *temp;
+	int retval;
+
+	/* We need to parse the latitude/longitude co-ordinates and location fields  */
+	sptr = (char *) str;
+	while (*sptr != '\t')
+		sptr++;
+	temp = ++sptr;
+	while (*sptr != '\t')
+		sptr++;
+	len = sptr-temp;
+	lat = (char *) malloc (len + 1);
+	lat = strncpy (lat, temp, len);
+	lat [len] = '\0';
+	while (*sptr != '\t')
+		sptr++;
+	
+	loc = ++sptr;
+	while (!isspace (*sptr))
+		sptr++;
+	len = sptr - loc;
+	location = strncpy (location, loc, len);
+	location [len] = '\0';
+	
+	lon = lat + 1;
+	while (*lon != '+' && *lon != '-')
+		lon++;
+
+	if (parse_coord (lat, lon - lat, latitude_degrees, latitude_minutes, latitude_seconds) == 1 ||
+		       	parse_coord (lon, strlen (lon), longitude_degrees, longitude_minutes, longitude_seconds) 
+			== 1)
+			return 1;
+	
+	free (lat);
+
+	return 0;
+}
 
 /** This parses the zones.tab file containing the names and locations
    of the builtin timezones. It creates the builtin_timezones array
@@ -1526,8 +1603,8 @@ icaltimezone_parse_zone_tab		(void)
     char buf[1024];  /* Used to store each line of zones.tab as it is read. */
     char location[1024]; /* Stores the city name when parsing buf. */
     unsigned int filename_len;
-    int latitude_degrees, latitude_minutes, latitude_seconds;
-    int longitude_degrees, longitude_minutes, longitude_seconds;
+    int latitude_degrees = 0, latitude_minutes = 0, latitude_seconds = 0;
+    int longitude_degrees = 0, longitude_minutes = 0, longitude_seconds = 0;
     icaltimezone zone;
 
     icalerror_assert (builtin_timezones == NULL,
@@ -1535,17 +1612,26 @@ icaltimezone_parse_zone_tab		(void)
 
     builtin_timezones = icalarray_new (sizeof (icaltimezone), 32);
 
+#ifndef WIN32
+    filename_len = strlen ((char *) icaltzutil_get_zone_directory()) + strlen (ZONES_TAB_SYSTEM_FILENAME)
+	+ 2;
+#else    
     filename_len = strlen (get_zone_directory()) + strlen (ZONES_TAB_FILENAME)
 	+ 2;
+#endif    
 
     filename = (char*) malloc (filename_len);
     if (!filename) {
 	icalerror_set_errno(ICAL_NEWFAILED_ERROR);
 	return;
     }
-
+#ifndef WIN32
+    snprintf (filename, filename_len, "%s/%s", icaltzutil_get_zone_directory (),
+	      ZONES_TAB_SYSTEM_FILENAME);
+#else    
     snprintf (filename, filename_len, "%s/%s", get_zone_directory(),
 	      ZONES_TAB_FILENAME);
+#endif    
 
     fp = fopen (filename, "r");
     free (filename);
@@ -1557,6 +1643,7 @@ icaltimezone_parse_zone_tab		(void)
     while (fgets (buf, sizeof(buf), fp)) {
 	if (*buf == '#') continue;
 
+#ifdef WIN32	
 	/* The format of each line is: "latitude longitude location". */
 	if (sscanf (buf, "%4d%2d%2d %4d%2d%2d %s",
 		    &latitude_degrees, &latitude_minutes,
@@ -1567,6 +1654,15 @@ icaltimezone_parse_zone_tab		(void)
 	    fprintf (stderr, "Invalid timezone description line: %s\n", buf);
 	    continue;
 	}
+#else 
+	if (fetch_lat_long_from_string (buf, &latitude_degrees, &latitude_minutes, 
+				&latitude_seconds,
+				&longitude_degrees, &longitude_minutes, &longitude_seconds,
+				location)) {
+	    fprintf (stderr, "Invalid timezone description line: %s\n", buf);
+	    continue;
+	}
+#endif 	
 
 	icaltimezone_init (&zone);
 	zone.location = strdup (location);
@@ -1591,10 +1687,10 @@ icaltimezone_parse_zone_tab		(void)
 
 	icalarray_append (builtin_timezones, &zone);
 
-#if 0
+#if 0	
 	printf ("Found zone: %s %f %f\n",
 		location, zone.latitude, zone.longitude);
-#endif
+#endif	
     }
 
     fclose (fp);
@@ -1605,16 +1701,18 @@ icaltimezone_parse_zone_tab		(void)
 static void
 icaltimezone_load_builtin_timezone	(icaltimezone	*zone)
 {
-    char *filename;
-    unsigned int filename_len;
-    FILE *fp;
-    icalparser *parser;
     icalcomponent *comp, *subcomp;
 
 	    /* If the location isn't set, it isn't a builtin timezone. */
     if (!zone->location || !zone->location[0])
 	return;
 
+#ifdef WIN32
+    char *filename;
+    unsigned int filename_len;
+    FILE *fp;
+    icalparser *parser;
+
     filename_len = strlen (get_zone_directory()) + strlen (zone->location) + 6;
 
     filename = (char*) malloc (filename_len);
@@ -1639,15 +1737,19 @@ icaltimezone_load_builtin_timezone	(icaltimezone	*zone)
 	fprintf(stderr, "** WARNING ** %s: %d %s\n", __FILE__, __LINE__, icalerror_strerror(icalerrno));
 	*/
 
-    parser = icalparser_new ();
+	parser = icalparser_new ();
 	icalparser_set_gen_data (parser, fp);
 	comp = icalparser_parse (parser, icaltimezone_load_get_line_fn);
-    icalparser_free (parser);
+    	icalparser_free (parser);
 	fclose (fp);
-
-    /* Find the VTIMEZONE component inside the VCALENDAR. There should be 1. */
-    subcomp = icalcomponent_get_first_component (comp,
+    
+	/* Find the VTIMEZONE component inside the VCALENDAR. There should be 1. */
+    	subcomp = icalcomponent_get_first_component (comp,
 						 ICAL_VTIMEZONE_COMPONENT);
+#else
+	subcomp = icaltzutil_fetch_timezone (zone->location);
+#endif	
+
     if (!subcomp) {
 	icalerror_set_errno(ICAL_PARSE_ERROR);
 	return;
@@ -1655,9 +1757,10 @@ icaltimezone_load_builtin_timezone	(icaltimezone	*zone)
 
     icaltimezone_get_vtimezone_properties (zone, subcomp);
 
-	icalcomponent_remove_component(comp,subcomp);
-
-	icalcomponent_free(comp);
+#ifdef WIN32   
+    icalcomponent_remove_component(comp,subcomp);
+    icalcomponent_free(comp);
+#endif    
 
 }
 
