@@ -23,20 +23,20 @@
 #include <config.h>
 #endif
 
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <unistd.h>
-#include <fcntl.h>
-#include <errno.h>
-#include <string.h>
-#include <stdlib.h>
 #include <ctype.h>
+#include <errno.h>
+#include <fcntl.h>
+#include <stdlib.h>
+#include <string.h>
+#include <unistd.h>
+#include <sys/stat.h>
+#include <sys/types.h>
 
 #include <glib.h>
+#include <glib/gi18n-lib.h>
 #include <glib/gstdio.h>
 
 #include "camel-file-utils.h"
-#include "camel-i18n.h"
 #include "camel-mime-message.h"
 #include "camel-operation.h"
 #include "camel-private.h"
@@ -55,6 +55,7 @@ static CamelMessageInfo * message_info_new_from_header(CamelFolderSummary *, str
 static CamelMessageInfo * message_info_new_from_parser(CamelFolderSummary *, CamelMimeParser *);
 static CamelMessageInfo * message_info_load (CamelFolderSummary *, FILE *);
 static int		  message_info_save (CamelFolderSummary *, FILE *, CamelMessageInfo *);
+static int 		  meta_message_info_save(CamelFolderSummary *s, FILE *out_meta, FILE *out, CamelMessageInfo *mi);
 /*static void		  message_info_free (CamelFolderSummary *, CamelMessageInfo *);*/
 
 static char *mbox_summary_encode_x_evolution (CamelLocalSummary *cls, const CamelLocalMessageInfo *mi);
@@ -154,6 +155,7 @@ camel_mbox_summary_class_init(CamelMboxSummaryClass *klass)
 	sklass->message_info_new_from_parser = message_info_new_from_parser;
 	sklass->message_info_load = message_info_load;
 	sklass->message_info_save = message_info_save;
+	sklass->meta_message_info_save = meta_message_info_save;
 	/*sklass->message_info_free = message_info_free;*/
 
 	sklass->info_set_user_flag = mbox_info_set_user_flag;
@@ -319,6 +321,7 @@ message_info_new_from_header(CamelFolderSummary *s, struct _camel_header_raw *h)
 
 		if (add&1) {
 			mi->info.info.flags |= CAMEL_MESSAGE_FOLDER_FLAGGED | CAMEL_MESSAGE_FOLDER_NOXEV;
+			g_free (mi->info.info.uid);
 			mi->info.info.uid = camel_folder_summary_next_uid_string(s);
 		} else {
 			camel_folder_summary_set_uid(s, strtoul(camel_message_info_uid(mi), NULL, 10));
@@ -381,6 +384,20 @@ error:
 	return NULL;
 }
 
+static int
+meta_message_info_save(CamelFolderSummary *s, FILE *out_meta, FILE *out, CamelMessageInfo *mi)
+{
+	CamelMboxMessageInfo *mbi = (CamelMboxMessageInfo *)mi;
+
+	io(printf("saving mbox message info\n"));
+
+	if (((CamelFolderSummaryClass *)camel_mbox_summary_parent)->meta_message_info_save(s, out_meta, out, mi) == -1
+	    || camel_file_util_encode_off_t(out_meta, mbi->frompos) == -1)
+		return -1;
+
+	return 0;
+}
+
 static int
 message_info_save(CamelFolderSummary *s, FILE *out, CamelMessageInfo *mi)
 {
@@ -496,7 +513,7 @@ summary_update(CamelLocalSummary *cls, off_t offset, CamelFolderChangeInfo *chan
 	
 	/* update the file size/mtime in the summary */
 	if (ok != -1) {
-		if (stat(cls->folder_path, &st) == 0) {
+		if (g_stat(cls->folder_path, &st) == 0) {
 			camel_folder_summary_touch(s);
 			mbs->folder_size = st.st_size;
 			s->time = st.st_mtime;
@@ -520,7 +537,7 @@ mbox_summary_check(CamelLocalSummary *cls, CamelFolderChangeInfo *changes, Camel
 	d(printf("Checking summary\n"));
 
 	/* check if the summary is up-to-date */
-	if (stat(cls->folder_path, &st) == -1) {
+	if (g_stat(cls->folder_path, &st) == -1) {
 		camel_folder_summary_clear(s);
 		camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
 				      _("Cannot check folder: %s: %s"),
@@ -599,7 +616,7 @@ mbox_summary_sync_full(CamelMboxSummary *mbs, gboolean expunge, CamelFolderChang
 
 	tmpname = g_alloca (strlen (cls->folder_path) + 5);
 	sprintf (tmpname, "%s.tmp", cls->folder_path);
-	d(printf("Writing tmp file to %s\n", tmpname));
+	d(printf("Writing temporary file to %s\n", tmpname));
 	fdout = g_open(tmpname, O_LARGEFILE|O_WRONLY|O_CREAT|O_TRUNC|O_BINARY, 0600);
 	if (fdout == -1) {
 		camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
@@ -623,9 +640,9 @@ mbox_summary_sync_full(CamelMboxSummary *mbs, gboolean expunge, CamelFolderChang
 	}
 
 	if (close(fdout) == -1) {
-		g_warning("Cannot close tmp folder: %s", strerror (errno));
+		g_warning("Cannot close temporary folder: %s", strerror (errno));
 		camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
-				      _("Could not close temp folder: %s"),
+				      _("Could not close temporary folder: %s"),
 				      g_strerror (errno));
 		fdout = -1;
 		goto error;
@@ -866,7 +883,7 @@ mbox_summary_sync(CamelLocalSummary *cls, gboolean expunge, CamelFolderChangeInf
 	if (ret == -1)
 		return -1;
 
-	if (stat(cls->folder_path, &st) == -1) {
+	if (g_stat(cls->folder_path, &st) == -1) {
 		camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
 				      _("Unknown error: %s"), g_strerror (errno));
 		return -1;
@@ -995,9 +1012,9 @@ camel_mbox_summary_sync_mbox(CamelMboxSummary *cls, guint32 flags, CamelFolderCh
 			}
 #endif
 			if (len == -1) {
-				d(printf("Error writing to tmp mailbox\n"));
+				d(printf("Writing to temporary mailbox failed\n"));
 				camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
-						      _("Error writing to temp mailbox: %s"),
+						      _("Writing to temporary mailbox failed: %s"),
 						      g_strerror (errno));
 				goto error;
 			}
@@ -1011,10 +1028,10 @@ camel_mbox_summary_sync_mbox(CamelMboxSummary *cls, guint32 flags, CamelFolderCh
 		if (info) {
 			d(printf("looking for message content to copy across from %d\n", (int)camel_mime_parser_tell(mp)));
 			while (camel_mime_parser_step(mp, &buffer, &len) == CAMEL_MIME_PARSER_STATE_PRE_FROM) {
-				/*d(printf("copying mbox contents to tmp: '%.*s'\n", len, buffer));*/
+				/*d(printf("copying mbox contents to temporary: '%.*s'\n", len, buffer));*/
 				if (write(fdout, buffer, len) != len) {
 					camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
-							      _("Writing to tmp mailbox failed: %s: %s"),
+							      _("Writing to temporary mailbox failed: %s: %s"),
 							      ((CamelLocalSummary *)cls)->folder_path,
 							      g_strerror (errno));
 					goto error;
@@ -1023,7 +1040,7 @@ camel_mbox_summary_sync_mbox(CamelMboxSummary *cls, guint32 flags, CamelFolderCh
 
 			if (write(fdout, "\n", 1) != 1) {
 				camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
-						      _("Error writing to temp mailbox: %s"),
+						      _("Writing to temporary mailbox failed: %s"),
 						      g_strerror (errno));
 				goto error;
 			}
