@@ -27,33 +27,35 @@
 #include <config.h>
 #endif
 
-#include <sys/types.h>
 #include <errno.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
-#include <errno.h>
+#include <sys/types.h>
+#include <ctype.h>
 
-#include "camel-operation.h"
+#include <glib/gi18n-lib.h>
+
+#include <libedataserver/md5-utils.h>
 
-#include "camel-pop3-store.h"
-#include "camel-pop3-folder.h"
-#include "camel-stream-buffer.h"
-#include "camel-session.h"
-#include "camel-exception.h"
-#include "camel-url.h"
-#include "libedataserver/md5-utils.h"
-#include "camel-pop3-engine.h"
-#include "camel-sasl.h"
 #include "camel-data-cache.h"
-#include "camel-tcp-stream.h"
+#include "camel-exception.h"
+#include "camel-net-utils.h"
+#include "camel-operation.h"
+#include "camel-pop3-engine.h"
+#include "camel-pop3-folder.h"
+#include "camel-pop3-store.h"
+#include "camel-sasl.h"
+#include "camel-session.h"
+#include "camel-stream-buffer.h"
 #include "camel-tcp-stream-raw.h"
+#include "camel-tcp-stream.h"
+#include "camel-url.h"
+
 #ifdef HAVE_SSL
 #include "camel-tcp-stream-ssl.h"
 #endif
-#include "camel-i18n.h"
-#include "camel-net-utils.h"
 
 /* Specified in RFC 1939 */
 #define POP3_PORT "110"
@@ -154,6 +156,7 @@ connect_to_server (CamelService *service, struct addrinfo *ai, int ssl_mode, Cam
 	guint32 flags = 0;
 	int clean_quit = TRUE;
 	int ret;
+	const gchar *delete_days;
 	
 	if (ssl_mode != MODE_CLEAR) {
 #ifdef HAVE_SSL
@@ -176,7 +179,7 @@ connect_to_server (CamelService *service, struct addrinfo *ai, int ssl_mode, Cam
 	if ((ret = camel_tcp_stream_connect ((CamelTcpStream *) tcp_stream, ai)) == -1) {
 		if (errno == EINTR)
 			camel_exception_set (ex, CAMEL_EXCEPTION_USER_CANCEL,
-					     _("Connection cancelled"));
+					     _("Connection canceled"));
 		else
 			camel_exception_setv (ex, CAMEL_EXCEPTION_SERVICE_UNAVAILABLE,
 					      _("Could not connect to %s: %s"),
@@ -197,6 +200,9 @@ connect_to_server (CamelService *service, struct addrinfo *ai, int ssl_mode, Cam
 	if (camel_url_get_param (service->url, "disable_extensions"))
 		flags |= CAMEL_POP3_ENGINE_DISABLE_EXTENSIONS;
 	
+	if ((delete_days = (gchar *) camel_url_get_param(service->url,"delete_after"))) 
+		store->delete_after =  atoi(delete_days);
+	
 	if (!(store->engine = camel_pop3_engine_new (tcp_stream, flags))) {
 		camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
 				      _("Failed to read a valid greeting from POP server %s"),
@@ -406,9 +412,9 @@ try_sasl(CamelPOP3Store *store, const char *mech, CamelException *ex)
 	while (1) {
 		if (camel_pop3_stream_line(stream, &line, &len) == -1)
 			goto ioerror;
-		if (strncmp(line, "+OK", 3) == 0)
+		if (strncmp((char *) line, (char *) "+OK", 3) == 0)
 			break;
-		if (strncmp(line, "-ERR", 4) == 0) {
+		if (strncmp((char *) line, (char *) "-ERR", 4) == 0) {
 			camel_exception_setv (ex, CAMEL_EXCEPTION_SERVICE_CANT_AUTHENTICATE,
 					      _("SASL `%s' Login failed for POP server %s: %s"),
 					      mech, CAMEL_SERVICE (store)->url->host, line);
@@ -416,9 +422,9 @@ try_sasl(CamelPOP3Store *store, const char *mech, CamelException *ex)
 		}
 		/* If we dont get continuation, or the sasl object's run out of work, or we dont get a challenge,
 		   its a protocol error, so fail, and try reset the server */
-		if (strncmp(line, "+ ", 2) != 0
+		if (strncmp((char *) line, (char *) "+ ", 2) != 0
 		    || camel_sasl_authenticated(sasl)
-		    || (resp = camel_sasl_challenge_base64(sasl, line+2, ex)) == NULL) {
+		    || (resp = (unsigned char *) camel_sasl_challenge_base64(sasl, (const char *) line+2, ex)) == NULL) {
 			camel_stream_printf((CamelStream *)stream, "*\r\n");
 			camel_pop3_stream_line(stream, &line, &len);
 			camel_exception_setv (ex, CAMEL_EXCEPTION_SERVICE_CANT_AUTHENTICATE,
@@ -438,7 +444,7 @@ try_sasl(CamelPOP3Store *store, const char *mech, CamelException *ex)
 	
  ioerror:
 	if (errno == EINTR) {
-		camel_exception_set (ex, CAMEL_EXCEPTION_USER_CANCEL, _("Cancelled"));
+		camel_exception_set (ex, CAMEL_EXCEPTION_USER_CANCEL, _("Canceled"));
 	} else {
 		camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
 				      _("Failed to authenticate on POP server %s: %s"),
@@ -485,6 +491,21 @@ pop3_try_authenticate (CamelService *service, gboolean reprompt, const char *err
 	} else if (strcmp(service->url->authmech, "+APOP") == 0 && store->engine->apop) {
 		char *secret, md5asc[33], *d;
 		unsigned char md5sum[16], *s;
+
+		d = store->engine->apop;
+
+		while (*d != '\0') {
+			if (!isascii((int)*d)) {
+
+				/* README for Translators: The string APOP should not be translated */
+				camel_exception_setv (ex, CAMEL_EXCEPTION_SERVICE_URL_INVALID,
+						_("Unable to connect to POP server %s:	Invalid APOP ID received. Impersonation attack suspected. Please contact your admin."),
+						CAMEL_SERVICE (store)->url->host);
+
+				return FALSE;
+			}
+			d++;
+		}
 		
 		secret = g_alloca(strlen(store->engine->apop)+strlen(service->url->passwd)+1);
 		sprintf(secret, "%s%s",  store->engine->apop, service->url->passwd);
@@ -519,7 +540,7 @@ pop3_try_authenticate (CamelService *service, gboolean reprompt, const char *err
 	
 	if (status == -1) {
 		if (errno == EINTR) {
-			camel_exception_set (ex, CAMEL_EXCEPTION_USER_CANCEL, _("Cancelled"));
+			camel_exception_set (ex, CAMEL_EXCEPTION_USER_CANCEL, _("Canceled"));
 		} else {
 			camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
 					      _("Unable to connect to POP server %s.\n"
